package com.qcg.comp.client;

import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.URL;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;

import javax.security.auth.login.CredentialException;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.datatype.DatatypeConfigurationException;

import org.globus.gsi.GlobusCredential;
import org.gridforum.jgss.ExtendedGSSManager;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;

import com.qcg.comp.sdk.ARES;
import com.qcg.comp.sdk.QCGFactory;
import com.qcg.comp.sdk.QCGManagement;
import com.qcg.comp.sdk.QCGRsync;
import com.qcg.comp.sdk.QCGStaging;
import com.qcg.comp.sdk.ardl.ARDL;
import com.qcg.comp.sdk.exceptions.CantApplyOperationToCurrentStateException;
import com.qcg.comp.sdk.exceptions.DeleteOnTerminationException;
import com.qcg.comp.sdk.exceptions.FileNotFoundException;
import com.qcg.comp.sdk.exceptions.InvalidRequestMessageException;
import com.qcg.comp.sdk.exceptions.NoMatchingResourcesException;
import com.qcg.comp.sdk.exceptions.NoSlotsAvailableException;
import com.qcg.comp.sdk.exceptions.NotAuthorizedException;
import com.qcg.comp.sdk.exceptions.StagingException;
import com.qcg.comp.sdk.exceptions.UnknownActivityIdentifierException;
import com.qcg.comp.sdk.exceptions.UnknownReservationIdentifierException;
import com.qcg.comp.sdk.exceptions.UnsupportedFeatureException;
import com.qcg.comp.sdk.jsdl.JSDL;
import com.qcg.comp.sdk.types.ActivityEndpointReference;
import com.qcg.comp.sdk.types.ReservationEndpointReference;
import com.qcg.comp.stubs.ares.ARESReservationStatusType;
import com.qcg.comp.stubs.ares.ObjectFactory;
import com.qcg.comp.stubs.bes.factory.FactoryResourceAttributesDocumentType;
import com.qcg.comp.stubs.rsync.IOFault;
import com.qcg.comp.stubs.rsync.InternalFault;
import com.qcg.comp.stubs.rsync.InvalidPathFault;
import com.qcg.comp.stubs.rsync.RsyncFault;
import com.qcg.comp.stubs.staging.FileNotFoundFault;
import com.qcg.comp.stubs.staging.NotAuthorizedFault;
import com.qcg.comp.stubs.staging.StagingFault;
import com.qcg.core.sdk.attachments.SimpleFileStagingHandler;
import com.qcg.core.sdk.security.AuthenticationModule;
import com.qcg.core.sdk.security.anonymous.AnonymousAuthentication;
import com.qcg.core.sdk.security.gsi.GSIAuthentication;
import com.qcg.core.sdk.security.ssl.SSLAuthentication;
import com.qcg.core.sdk.security.wsse.WSSEAuthentication;
import com.qcg.core.sdk.security.wsse.username.UsernameToken;
import com.qcg.core.sdk.utils.EndpointReferenceUtils;

public class QCGComputingClient {

	public static final String DEFAULT_SERVICE_URL = "http://localhost:19000/";
	
	static boolean debug = Boolean.parseBoolean(System.getProperty("qcg.debug", "false"));
	boolean wsa = Boolean.parseBoolean(System.getProperty("qcg.wsa", "true"));
	String serviceURL = System.getProperty("qcg.comp.service.url", DEFAULT_SERVICE_URL);
	String username = System.getProperty("qcg.comp.username");
	String password = System.getProperty("qcg.comp.password");
	String caFile = System.getProperty("qcg.comp.ca.file");
	String caPath = System.getProperty("qcg.comp.ca.path");
	String trustedDN = System.getProperty("qcg.comp.trustedDN");
	String clientCertKey = System.getProperty("qcg.comp.client.certkey");
	String clientCertPass = System.getProperty("qcg.comp.client.keypass");
	int repeat = Integer.parseInt(System.getProperty("qcg.repeat.count", "1"));
	
	String jsdlContent;
	String jsdlFile;
	String resUUID;
	String resEprFile;
	String actUUID;
	String actEprFile;
	String resdocContent;
	String resdocFile;
	ArrayList<String> rsyncLocal = new ArrayList<String>();
	ArrayList<String> rsyncExclude = new ArrayList<String>();
	ACTION action = ACTION.n_NONE;
	
	QCGFactory besFactory;
	QCGManagement besManagement;
	ARES ares;
	QCGStaging staging;
	QCGRsync rsync;
	
	public QCGComputingClient(String[] args) {
		
	    	boolean parseArgs = true;
	    
		String actionStr = args[0].substring(1,2);								
		
		for (int i=1; i < args.length; i++) {	
		    
		    	if(parseArgs) {
        			if (args[i].equals("-j"))
        				jsdlContent = args[++i];
        			
        			else if (args[i].equals("-J"))
        				jsdlFile = args[++i];
        			
        			else if (args[i].equals("-r"))
        				resUUID = args[i+1];
        			
        			else if (args[i].equals("-R"))
        				resEprFile = args[i+1];
        			
        			else if (args[i].equals("-a"))
        				actUUID = args[i+1];
        			
        			else if (args[i].equals("-A"))
        				actEprFile = args[i+1];
        			
        			else if (args[i].equals("-d"))
        				resdocContent = args[i+1];
        			
        			else if (args[i].equals("-D"))
        				resdocFile = args[i+1];
        			
        			else if (args[i].equals("-X"))
        			    	rsyncExclude.add(args[++i]);
        			
        			else if (args[i].equals("--"))
        			    	parseArgs = false;
        			
        			else
        			    	rsyncLocal.add(args[i]);
		    	}
		    	else
			    	rsyncLocal.add(args[i]);
		}
		
		//simple vaidation:
		if (jsdlContent != null && jsdlFile!= null)
			throw new IllegalArgumentException("only one of the -j/-J options may be given");
		
		if (resUUID != null && resEprFile!= null)
			throw new IllegalArgumentException("only one of the -r/-R options may be given");
		
		if (actUUID != null && actEprFile!= null)
			throw new IllegalArgumentException("only one of the -a/-A options may be given");
		
		if (resdocContent != null && resdocFile!= null)
			throw new IllegalArgumentException("only one of the -d/-D options may be given");
		
		for (ACTION a : ACTION.values()) {
			if (a.name().startsWith(actionStr)) {
				action = a;
				break;
			}
		}
		
		if (action == ACTION.n_NONE) {
			throw new IllegalArgumentException("No action specified");
		}
	}
	
	public static void initGSICredentials() {
		ExtendedGSSManager manager = (ExtendedGSSManager) ExtendedGSSManager
				.getInstance();

		try {
			manager.createCredential(GSSCredential.INITIATE_AND_ACCEPT);
		} catch (GSSException e) {
			org.globus.tools.ProxyInit.main(new String[] {});
		}
	}
	
	/*
	public static void loadCredential() throws CredentialException
	{
		int tryCount = 0;
		ExtendedGSSManager manager = (ExtendedGSSManager)ExtendedGSSManager.getInstance();
		
		do
		{
			try
			{
				String proxy = System.getenv("X509_USER_PROXY");
				logDebug("proxy = " + proxy);
				
				if( proxy!=null && !proxy.isEmpty())
				{
					GlobusCredential gcredential = new GlobusCredential( proxy);
					credential = new GlobusGSSCredentialImpl(gcredential, GSSCredential.INITIATE_AND_ACCEPT);
				}
				else
					credential = manager.createCredential( GSSCredential.INITIATE_AND_ACCEPT);
				
				break;
			}
			catch( Exception e)
			{
				if( tryCount == 0)
				{
					ProxyInit.run();
					tryCount++;
					continue;
				}
					
				throw new CredentialException( e.getMessage(), e);
			}
		}
		while( tryCount < 2);
	}
	*/
	
	public void execute() throws Exception {
		
		if (debug)
			System.setProperty("com.sun.xml.ws.transport.http.client.HttpTransportPipe.dump", "true");
		//setup ports:
		
		AuthenticationModule transportAM = null;
		AuthenticationModule messageAM =  null;
		
		
		System.err.println("Service URL = " + serviceURL);
		
		boolean gsiEnabled = (serviceURL.startsWith("httpg://")); /* ugly hack on httpg */
		if (gsiEnabled)
			serviceURL = serviceURL.replaceFirst("httpg", "https");
		
		URL sURL = new URL(serviceURL);
		
		String host = sURL.getHost();
		int port = sURL.getPort();
		String protocol = sURL.getProtocol();
		
		if (protocol.equals("http")) {
			//http
			transportAM =  new AnonymousAuthentication();
		} else if (protocol.equals("https") && !gsiEnabled) {
			//SSL
			if (clientCertKey != null) {
				//mutual
				transportAM = new SSLAuthentication(caFile, caPath, clientCertKey, clientCertPass, trustedDN);
			} else {
				transportAM = new SSLAuthentication(caFile, caPath, trustedDN);
			}
			 
		} else if (protocol.equals("https") && gsiEnabled) {
			//GSI
			//
			if (System.getenv("X509_USER_PROXY") != null && !System.getenv("X509_USER_PROXY").isEmpty()) {
				transportAM = new GSIAuthentication(System.getenv("X509_USER_PROXY"), trustedDN);
			} else {
				initGSICredentials();
				transportAM = new GSIAuthentication(trustedDN);
			}
		}else {
			throw new IllegalArgumentException("Unknown protocol: " + protocol);
		}
		
		if (username != null) {
			messageAM =  new WSSEAuthentication(new UsernameToken(username, password, false, false));
		}
		
		besFactory = new QCGFactory(host, port, transportAM,messageAM, wsa);
		besManagement =  new QCGManagement(host, port, transportAM, messageAM, wsa);
		ares = new ARES(host, port, transportAM, messageAM, wsa);
		staging = new QCGStaging(host, port, transportAM, messageAM, wsa, new SimpleFileStagingHandler());
		rsync = new QCGRsync(host, port, transportAM, messageAM, wsa);
		
		for (int i =0; i < repeat; i++) {
			switch (action) {
	
				case c_CREATE:
					
					if (jsdlContent!= null || jsdlFile != null) {
						createActivity();
					} else if (resdocContent!=null || resdocFile != null) {
						createReservation();
					} else {
						throw new IllegalArgumentException("Missing argument");
					}
					
					break;
				case s_GET_STATUS:
					
					if (actUUID!= null || actEprFile != null)
						getActivityStatus();
					else if (resUUID!=null || resEprFile != null)
						getReservationStatus();
					else
						throw new IllegalArgumentException("Missing argument");
					
					
					break;
					
				case t_TERMINATE:
					
					if (actUUID!= null || actEprFile != null)
						terminateActivity();
					else if (resUUID!=null || resEprFile != null)
						terminateReservation();
					else
						throw new IllegalArgumentException("Missing argument");
					
					break;
					
				case e_ENABLE_SUB:
					
					enableSub();
					
					break;
				case d_DISABLE_SUB:
					
					disableSub();
					
					break;
				case g_GET_DOC:
					
					if (actUUID!= null || actEprFile != null)
						getActivityDoc();
					else if (resUUID!=null || resEprFile != null)
						getReservationDoc();
					else
						throw new IllegalArgumentException("Missing argument");
					
					break;
				case G_GET_FACT_DOC:
					getFactoryDoc();
					break;
				case l_LIST_RES:
					listReservations();
					break;
				case i_STAGE_IN:
					stageIn();
					break;
				case o_STAGE_OUT:
					stageOut();
					break;
				case L_RSYNC_LOCAL:
				    	rsyncLocal();
				    	break;
				default:
					usage();
					break;
				}
		}
	}
	
	private void rsyncLocal() throws NoSuchAlgorithmException, InternalFault, InvalidPathFault, DatatypeConfigurationException, IOException, IOFault, RsyncFault, JAXBException, FileNotFoundFault, NotAuthorizedFault, StagingFault {
	    JSDL jsdl = null;
		
	    log(jsdlContent);
	    
	    if (jsdlContent != null)
		jsdl = new JSDL(new StringReader(jsdlContent));
	    else if (jsdlFile != null)
		jsdl = new JSDL(new FileReader(jsdlFile));
	    else
	    	throw new IllegalArgumentException("Missing JSDL argument");
	    	
	    for(String local : rsyncLocal) {
		System.out.println("Syncing from "+local+" to "+serviceURL+":"+jsdl.getWorkingDirectory());
		rsync.rsyncLocalToRemote(jsdl, local, rsyncExclude);
	    }
	}

	private void stageOut() throws JAXBException,  IOException, DeleteOnTerminationException, FileNotFoundException, NotAuthorizedException, StagingException {
		JSDL jsdl = null;
		
		if (jsdlContent != null)
			jsdl = new JSDL(new StringReader(jsdlContent));
		else if (jsdlFile != null)
			jsdl = new JSDL(new FileReader(jsdlFile));
		else
			throw new IllegalArgumentException("Missing JSDL argument");
		
		staging.stageOutFiles(jsdl);
	}

	private void stageIn() throws JAXBException, IOException, FileNotFoundException, NotAuthorizedException, StagingException {
		JSDL jsdl = null;
		
		if (jsdlContent != null)
			jsdl = new JSDL(new StringReader(jsdlContent));
		else if (jsdlFile != null)
			jsdl = new JSDL(new FileReader(jsdlFile));
		else
			throw new IllegalArgumentException("Missing JSDL argument");
		
		staging.stageInFiles(jsdl);
	}

	private void listReservations() throws NotAuthorizedException {
		List<ReservationEndpointReference> activeRes = ares.getActiveReservations();
		int i = 0;
		
		for (ReservationEndpointReference rEpr : activeRes) {
			i++;
			System.err.println("ActiveReservation[" + i + "]=" + rEpr.getReservationUUID());
		}
	}

	private void getFactoryDoc() throws JAXBException {
		System.err.println("FactoryAttributes[" + serviceURL + "]:");
		
		FactoryResourceAttributesDocumentType frad =  besFactory.getFactoryAttributesDocument();
		
		Marshaller m = besFactory.getJaxbContext().createMarshaller();
		StringWriter sw  = new StringWriter(512);
		m.setProperty("jaxb.formatted.output",new Boolean(true)); 
		m.marshal((new com.qcg.comp.stubs.bes.factory.ObjectFactory()).createFactoryResourceAttributesDocument(frad), sw);
		System.err.println(sw.getBuffer().toString());
	}

	private void getReservationDoc() throws IOException, NotAuthorizedException, UnknownReservationIdentifierException {
		ReservationEndpointReference rer = null;
		
		if (resUUID != null) {
			rer = new ReservationEndpointReference(serviceURL, resUUID);
		} else if (resEprFile != null){
			rer = new ReservationEndpointReference(EndpointReferenceUtils.loadFromFile(resEprFile));
		} else {
			throw new IllegalArgumentException("Missing reservation identifier");
		}
		
		System.err.println("reservation documents[" + rer.getReservationUUID() + "]:");
		System.err.println(ares.getReservationDocument(rer));		
	}

	private void getActivityDoc() throws IOException, NotAuthorizedException, UnknownActivityIdentifierException {
		ActivityEndpointReference aer = null;
		
		if (actUUID != null) {
			aer = new ActivityEndpointReference(serviceURL, actUUID);
		} else if (actEprFile != null){
			aer = new ActivityEndpointReference(EndpointReferenceUtils.loadFromFile(actEprFile));
		} else {
			throw new IllegalArgumentException("Missing activity identifier");
		}
		
		System.err.println("activity documents[" + aer.getActivityUUID() + "]:");
		System.err.println(besFactory.getActivityDocuments(aer));
	}

	private void disableSub() throws NotAuthorizedException {
		besManagement.stopAcceptingNewActivites();
	}

	private void enableSub() throws NotAuthorizedException {
		besManagement.startAcceptingNewActivites();
	}

	private void createReservation() throws JAXBException, IOException, InvalidRequestMessageException, NoMatchingResourcesException, NoSlotsAvailableException, NotAuthorizedException, UnsupportedFeatureException {
		ARDL resDoc = null;
		
		if (resdocContent != null)
			resDoc = new ARDL(new StringReader(resdocContent));
		else if (resdocFile != null)
			resDoc = new ARDL(new FileReader(resdocFile));
		else
			throw new IllegalArgumentException("Missing ARDL argument");
		
		ReservationEndpointReference epr = ares.createReservation(resDoc);
		log("Reservation created.");
		
		if (resEprFile != null) {
			EndpointReferenceUtils.saveToFile(epr.getEndpointReference(), resEprFile);
			log("Reservation EPR saved in " + resEprFile);
		} else {
			log("Reservation UUID: " + epr.getReservationUUID());
		}
	}
	
	private void terminateReservation() throws IOException, NotAuthorizedException, UnknownReservationIdentifierException {
		ReservationEndpointReference rer = null;
		
		if (resUUID != null) {
			rer = new ReservationEndpointReference(serviceURL, resUUID);
		} else if (resEprFile != null){
			rer = new ReservationEndpointReference(EndpointReferenceUtils.loadFromFile(resEprFile));
		} else {
			throw new IllegalArgumentException("Missing reservation identifier");
		}
		
		System.err.println("terminating reservation[" + rer.getReservationUUID() + "]");
		ares.terminateReservation(rer);
	}

	private void terminateActivity() throws IOException, CantApplyOperationToCurrentStateException, NotAuthorizedException, UnknownActivityIdentifierException {
		ActivityEndpointReference aer = null;
		
		if (actUUID != null) {
			aer = new ActivityEndpointReference(serviceURL, actUUID);
		} else if (actEprFile != null){
			aer = new ActivityEndpointReference(EndpointReferenceUtils.loadFromFile(actEprFile));
		} else {
			throw new IllegalArgumentException("Missing activity identifier");
		}
		
		System.err.println("terminating activity[" + aer.getActivityUUID() + "]");
		System.err.println("terminated = " + besFactory.terminateActivity(aer));
	}

	private void getActivityStatus() throws IOException, NotAuthorizedException, UnknownActivityIdentifierException {
		ActivityEndpointReference aer = null;
		
		if (actUUID != null) {
			aer = new ActivityEndpointReference(serviceURL, actUUID);
		} else if (actEprFile != null){
			aer = new ActivityEndpointReference(EndpointReferenceUtils.loadFromFile(actEprFile));
		} else {
			throw new IllegalArgumentException("Missing activity identifier");
		}
		
		System.err.println("Status of activity[" + aer.getActivityUUID() + "]: " + besFactory.getActivityStatus(aer));
	}
	
	private void getReservationStatus() throws IOException, JAXBException, NotAuthorizedException, UnknownReservationIdentifierException {
		ReservationEndpointReference rer = null;
		
		if (resUUID != null) {
			rer = new ReservationEndpointReference(serviceURL, resUUID);
		} else if (resEprFile != null){
			rer = new ReservationEndpointReference(EndpointReferenceUtils.loadFromFile(resEprFile));
		} else {
			throw new IllegalArgumentException("Missing reservation identifier");
		}
		
		ARESReservationStatusType rstat = ares.getReservationStatus(rer);
		
		System.err.println("Status of reservation[" + rer.getReservationUUID() + "]: ");
		
		Marshaller m = ares.getJaxbContext().createMarshaller();
		m.setProperty("jaxb.formatted.output", new Boolean(true)); 

		StringWriter sw = new StringWriter(512);
		m.marshal((new ObjectFactory()).createReservationStatus(rstat), sw);
		System.err.println(sw.getBuffer().toString());
	}

	private void createActivity() throws JAXBException, IOException, InvalidRequestMessageException, NotAuthorizedException, UnsupportedFeatureException {
		JSDL jsdl = null;
		ReservationEndpointReference resEpr = null;
		
		if (jsdlContent != null)
			jsdl = new JSDL(new StringReader(jsdlContent));
		else if (jsdlFile != null)
			jsdl = new JSDL(new FileReader(jsdlFile));
		else
			throw new IllegalArgumentException("Missing JSDL argument");
		
		if (resEprFile != null) {
			resEpr = new ReservationEndpointReference(new FileInputStream(resEprFile));
		} else if (resUUID != null) {
			resEpr = new ReservationEndpointReference(serviceURL, resUUID);
		}
		
		if (resEpr!= null)
			jsdl.setReservationIdentifier(resEpr);
		
		ActivityEndpointReference epr = besFactory.createActivity(jsdl);
		log("Activity created.");
		
		if (actEprFile != null) {
			EndpointReferenceUtils.saveToFile(epr.getEndpointReference(), actEprFile);
			log("Activity EPR saved in " + actEprFile);
		} else {
			log("Activity UUID: " + epr.getActivityUUID());
		}
	}

	public static void main(String[] args) {
		try {
			QCGComputingClient client = new QCGComputingClient(args);
			client.execute();
		} catch (IndexOutOfBoundsException ex) {
			log("Illegal usage!");
			usage();
		} catch (IllegalArgumentException ex) {
			log(ex.getMessage());
			usage();			
		} catch (Exception ex) {
			System.err.println(ex);
			if (debug)
				ex.printStackTrace();
		}
	}
	
	public static void usage() {
		log("Usage: java -jar QCGComputingSDK-2.2.jar OPTIONS");
		log("");
		log("BES-Factory port usage:");
		log("  -c (-j jsdl|-J jsdl.xml) [-r reservationUUID|-R reservation-epr.xml] [-A activity-epr.xml]");
		log("                                                    create activity, optionally save activity EPR to file");
		log("  -s (-a activityUUID|-A activity-epr.xml)          check status of the activity");
		log("  -g (-a activityUUID|-A activity-epr.xml)          get activity document");
		log("  -t (-a activityUUID|-A activity-epr.xml)          terminate activity");
		log("  -G                                                get factory document");						
		log("");
		log("BES-Management port usage:");
		log("  -e                                                enable submission");
		log("  -d                                                disable submission");
		log("");
		log("QCG-ARES port usage:");
		log("  -c (-d ardl|-D ardl.xml) [-R reservation-epr.xml]");
		log("                                                    create reservation, optionally save reservation EPR to file");
		log("  -s (-r reservationUUID|-R reservation-epr.xml)    check status of the reservation");
		log("  -g (-r reservationUUID|-R reservation-epr.xml)    get reservation document");
		log("  -t (-r reservationUUID|-R reservation-epr.xml)    terminate reservation");
		log("  -l                                                list active reservations");
		log("");
		log("QCG-Staging port usage:");
		log("  -i (-j jsdl|-J jsdl.xml)                          stage files in");
		log("  -o (-j jsdl|-J jsdl.xml)                          stage files out");
		log("");
		log("QCG-Rsync port usage:");
		log("  -L (-j jsdl|-J jsdl.xml)                          rsync files from local site (updates remote site)");
		log("");
		log("  -h                                                print this information");
		log("");
		log("Optional VM args:");
		log("  -Dqcg.comp.service.url=URL                       service url (default: " + DEFAULT_SERVICE_URL + ")");
		log("  -Dqcg.debug=true                                 turn on debugging (default: off)");
		log("  -Dqcg.wsa=false                                  turn off WS-Addressing headers (default: on)");
		log("  -Dqcg.comp.username=user                         username (WSSE UserName Token profile - HTTP/HTTPS)");
		log("  -Dqcg.comp.password=pass                         user's password (WSSE UserName Token profile - HTTP/HTTPS)");
		log("  -Dqcg.comp.ca.file=file                          path to CA certificate (HTTPS)");
		log("  -Dqcg.comp.ca.path=dir                           path to  directory with CA certificates (HTTPS) ");
		log("  -Dqcg.comp.trustedDN=\"C=PL..\"                  expected Distinguished Name of the service (HTTPS/HTTPG)");
		log("  -Dqcg.comp.client.certkey=file                   path to file with client key and certificate (HTTPS)");
		log("  -Dqcg.comp.client.keypass=pass                   key's passphrase (HTTPS)");



	}
	
	private enum ACTION { n_NONE, c_CREATE, s_GET_STATUS, t_TERMINATE, e_ENABLE_SUB, d_DISABLE_SUB, g_GET_DOC, G_GET_FACT_DOC, l_LIST_RES, i_STAGE_IN, o_STAGE_OUT, h_HELP, L_RSYNC_LOCAL} ;
	
	public static void log(String msg) {
		System.err.println(msg);
	}
	
	public static void logDebug(String msg) {
		if (debug)
			System.err.println(msg);
	}
}
