package com.qcg.comp.sdk;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;
import javax.xml.ws.Service;
import javax.xml.ws.WebServiceFeature;
import javax.xml.ws.handler.Handler;
import javax.xml.ws.wsaddressing.W3CEndpointReference;

import com.qcg.comp.sdk.exceptions.CantApplyOperationToCurrentStateException;
import com.qcg.comp.sdk.exceptions.InvalidRequestMessageException;
import com.qcg.comp.sdk.exceptions.NotAuthorizedException;
import com.qcg.comp.sdk.exceptions.UnknownActivityIdentifierException;
import com.qcg.comp.sdk.exceptions.UnsupportedFeatureException;
import com.qcg.comp.sdk.jsdl.JSDL;
import com.qcg.comp.sdk.jsdl.JSDLDataStaging;
import com.qcg.comp.sdk.types.ActivityDocument;
import com.qcg.comp.sdk.types.ActivityEndpointReference;
import com.qcg.comp.sdk.types.QCGActivityStatus;
import com.qcg.comp.stubs.bes.factory.ActivityDocumentType;
import com.qcg.comp.stubs.bes.factory.BESFactory;
import com.qcg.comp.stubs.bes.factory.BESFactoryPortType;
import com.qcg.comp.stubs.bes.factory.CantApplyOperationToCurrentStateFault;
import com.qcg.comp.stubs.bes.factory.CreateActivityResponseType;
import com.qcg.comp.stubs.bes.factory.CreateActivityType;
import com.qcg.comp.stubs.bes.factory.FactoryResourceAttributesDocumentType;
import com.qcg.comp.stubs.bes.factory.GetActivityDocumentResponseType;
import com.qcg.comp.stubs.bes.factory.GetActivityDocumentsResponseType;
import com.qcg.comp.stubs.bes.factory.GetActivityDocumentsType;
import com.qcg.comp.stubs.bes.factory.GetActivityStatusResponseType;
import com.qcg.comp.stubs.bes.factory.GetActivityStatusesResponseType;
import com.qcg.comp.stubs.bes.factory.GetActivityStatusesType;
import com.qcg.comp.stubs.bes.factory.GetFactoryAttributesDocumentResponseType;
import com.qcg.comp.stubs.bes.factory.GetFactoryAttributesDocumentType;
import com.qcg.comp.stubs.bes.factory.InvalidRequestMessageFault;
import com.qcg.comp.stubs.bes.factory.NotAcceptingNewActivitiesFault;
import com.qcg.comp.stubs.bes.factory.NotAuthorizedFault;
import com.qcg.comp.stubs.bes.factory.TerminateActivitiesResponseType;
import com.qcg.comp.stubs.bes.factory.TerminateActivitiesType;
import com.qcg.comp.stubs.bes.factory.TerminateActivityResponseType;
import com.qcg.comp.stubs.bes.factory.UnknownActivityIdentifierFault;
import com.qcg.comp.stubs.bes.factory.UnsupportedFeatureFault;
import com.qcg.comp.stubs.factory.ApplicationsType;
import com.qcg.comp.stubs.hpcp.bf.BasicFilterType;
import com.qcg.core.sdk.QCGCoreService;
import com.qcg.core.sdk.security.AuthenticationModule;

public class QCGFactory extends QCGCoreService {
	public static final String BES_FACTORY_NS = "http://schemas.ggf.org/bes/2006/08/bes-factory";
	protected BESFactoryPortType besFactory;
	
	public QCGFactory(String hostname, int port, 
			AuthenticationModule transportLayer,
			AuthenticationModule messageLayer,
			boolean useWSA) throws JAXBException {
		super(hostname, port, transportLayer, messageLayer, useWSA);
		
		besFactory = (BESFactoryPortType)stubs;
	}
	
	public QCGFactory(String hostname, int port, 
			AuthenticationModule transportLayer,
			AuthenticationModule messageLayer) throws JAXBException {
		super(hostname, port, transportLayer, messageLayer);
		
		besFactory = (BESFactoryPortType)stubs;
	}
	
	public QCGFactory(String hostname, int port, 
			AuthenticationModule transportLayer,
			boolean useWSA) throws JAXBException {
		super(hostname, port, transportLayer, useWSA);
		
		besFactory = (BESFactoryPortType)stubs;
	}
	
	public QCGFactory(String hostname, int port, 
			AuthenticationModule transportLayer) throws JAXBException {
		super(hostname, port, transportLayer);
		
		besFactory = (BESFactoryPortType)stubs;
	}
	@Override
	protected Service getService(){

		return new BESFactory(QCGFactory.class.getResource("/schemas/bes-factory.wsdl"), new QName(BES_FACTORY_NS, "BESFactory"));
	}
	
	@Override
	@SuppressWarnings("all")
	protected List<Handler> getHandlersChain() {
		return null;
	}

	@Override
	protected Object getPort(Service service, WebServiceFeature... features) {
		return ((BESFactory)service).getBESFactory(features);
	}
		
	public synchronized ActivityEndpointReference createActivity(JSDL jsdl) throws InvalidRequestMessageException, NotAuthorizedException, UnsupportedFeatureException {
		CreateActivityType parameters =  new CreateActivityType();
		JSDL jsdlCopy = null;
		
		try {
			jsdlCopy = (JSDL)jsdl.clone();
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
			throw new AssertionError();
		}
		
		List<JSDLDataStaging> toRemove = new LinkedList<JSDLDataStaging>();
		
		for (JSDLDataStaging ds:  jsdlCopy.getDataStaging()) {
			if (ds.getSourceURL() != null  && (!ds.getSourceURL().contains(":") || ds.getSourceURL().startsWith("cid:")))
				toRemove.add(ds);
			
			if (ds.getTargetURL() != null  && !ds.getTargetURL().contains(":"))
				toRemove.add(ds);
		}
		
		jsdlCopy.getDataStaging().removeAll(toRemove);
		
		parameters.setActivityDocument(new ActivityDocumentType());
		parameters.getActivityDocument().setJobDefinition(jsdlCopy.toJobDefinition()); 
		
		CreateActivityResponseType response;
		try {
			response = besFactory.createActivity(parameters);
		} catch (InvalidRequestMessageFault e) {
			throw new InvalidRequestMessageException(e);
		} catch (NotAcceptingNewActivitiesFault e) {
			throw new NotAuthorizedException(e);
		} catch (NotAuthorizedFault e) {
			throw new NotAuthorizedException(e);
		} catch (UnsupportedFeatureFault e) {
			throw new UnsupportedFeatureException(e);
		}
		
		return new ActivityEndpointReference(response.getActivityIdentifier());
	}
	
	public synchronized QCGActivityStatus getActivityStatus(ActivityEndpointReference activityIdentifier) throws NotAuthorizedException, UnknownActivityIdentifierException {
		
		return getActivityStatuses(getSingleElemList(activityIdentifier)).get(0);
	}
	
	public synchronized List<QCGActivityStatus> getActivityStatuses(List<ActivityEndpointReference> activityIdentifiers) throws NotAuthorizedException, UnknownActivityIdentifierException {
		GetActivityStatusesType parameters =  new GetActivityStatusesType();
		parameters.getActivityIdentifier().addAll(convert(activityIdentifiers));
		
		
		GetActivityStatusesResponseType response;
		try {
			response = besFactory.getActivityStatuses(parameters);
		} catch (NotAuthorizedFault e) {
			throw new NotAuthorizedException(e);
		} catch (UnknownActivityIdentifierFault e) {
			throw new UnknownActivityIdentifierException(e);
		}
		
		List<QCGActivityStatus> activitiesStatus = new ArrayList<QCGActivityStatus>(response.getResponse().size());
		
		for (GetActivityStatusResponseType activityStat : response.getResponse()) {
			activitiesStatus.add(new QCGActivityStatus(activityStat.getActivityIdentifier(), activityStat.getActivityStatus()));
		}
		
		return activitiesStatus;
	}
	
	public synchronized boolean terminateActivity(ActivityEndpointReference activityIdentifier) throws CantApplyOperationToCurrentStateException, NotAuthorizedException, UnknownActivityIdentifierException {
		
		return terminateActivites(getSingleElemList(activityIdentifier)).get(0);
	}
	
	public synchronized List<Boolean> terminateActivites(List<ActivityEndpointReference> activityIdentifiers) throws CantApplyOperationToCurrentStateException, NotAuthorizedException, UnknownActivityIdentifierException {
		TerminateActivitiesType parameters =  new TerminateActivitiesType();
		parameters.getActivityIdentifier().addAll(convert(activityIdentifiers));
		List<Boolean> termStatus = new ArrayList<Boolean>(activityIdentifiers.size());
		
		TerminateActivitiesResponseType response;
		try {
			response = besFactory.terminateActivities(parameters);
		} catch (CantApplyOperationToCurrentStateFault e) {
			throw new CantApplyOperationToCurrentStateException(e);
		} catch (NotAuthorizedFault e) {
			throw new NotAuthorizedException(e);
		} catch (UnknownActivityIdentifierFault e) {
			throw new UnknownActivityIdentifierException(e);
		}
		
		for (TerminateActivityResponseType res : response.getResponse()) {
				if (res.getFault() != null) {
					termStatus.add(new Boolean(false));
				} else if (res.isTerminated()) {
					termStatus.add(new Boolean(true));
				} else {
					termStatus.add(new Boolean(false));
				}
		}
		
		return termStatus;
		
	}
	
	public synchronized List<ActivityDocument> getActivitiesDocuments(List<ActivityEndpointReference> activitiesIdentifier) throws NotAuthorizedException, UnknownActivityIdentifierException {
		GetActivityDocumentsType parameters = new GetActivityDocumentsType();
		parameters.getActivityIdentifier().addAll(convert(activitiesIdentifier));
		
		GetActivityDocumentsResponseType response;
		try {
			response = besFactory.getActivityDocuments(parameters);
		} catch (NotAuthorizedFault e) {
			throw new NotAuthorizedException(e);
		} catch (UnknownActivityIdentifierFault e) {
			throw new UnknownActivityIdentifierException(e);
		}
		
		ArrayList<ActivityDocument> dList =  new ArrayList<ActivityDocument>();
		
		for ( GetActivityDocumentResponseType res : response.getResponse()) {
			dList.add(new ActivityDocument(new JSDL(res.getJobDefinition()), res.getLocalActivityId()));
		}
		
		return dList;
	}

	public synchronized ActivityDocument getActivityDocuments(ActivityEndpointReference activityIdentifier) throws NotAuthorizedException, UnknownActivityIdentifierException {
		
		return getActivitiesDocuments(getSingleElemList(activityIdentifier)).get(0);
	}
	
	public synchronized FactoryResourceAttributesDocumentType getFactoryAttributesDocument() {
		GetFactoryAttributesDocumentType parameters = new GetFactoryAttributesDocumentType();
		GetFactoryAttributesDocumentResponseType response = besFactory.getFactoryAttributesDocument(parameters);
		
		return response.getFactoryResourceAttributesDocument();
	}
	
	public synchronized static ApplicationsType getApplications(FactoryResourceAttributesDocumentType factoryDoc) {
		for (Object any : factoryDoc.getAny()) {
			if (any instanceof JAXBElement && ((JAXBElement) any).getDeclaredType().equals(ApplicationsType.class)) {
				return ((ApplicationsType)((JAXBElement) any).getValue());
			}
		}
		
		return null;
	}
	
	public synchronized FactoryResourceAttributesDocumentType getFactoryAttributesDocument(boolean returnActivities, boolean returnResources) {
		GetFactoryAttributesDocumentType parameters = new GetFactoryAttributesDocumentType();
		
		parameters.setBasicFilter(new BasicFilterType());
		parameters.getBasicFilter().setActivityReferences(returnActivities);
		parameters.getBasicFilter().setContainedResources(returnResources);
		
		GetFactoryAttributesDocumentResponseType response = besFactory.getFactoryAttributesDocument(parameters);
		
		return response.getFactoryResourceAttributesDocument();
	}
	
	protected synchronized List<ActivityEndpointReference> getSingleElemList(ActivityEndpointReference activityIdentifier) {

		List<ActivityEndpointReference> aiList = new ArrayList<ActivityEndpointReference>();
		aiList.add(activityIdentifier);
		
		return aiList;
	}
	
	protected synchronized List<W3CEndpointReference> convert(List<ActivityEndpointReference> activitiesIdentifier) {
		List<W3CEndpointReference> w3cList = new ArrayList<W3CEndpointReference>(activitiesIdentifier.size());
		
		for (ActivityEndpointReference aer : activitiesIdentifier) {
			w3cList.add(aer.getEndpointReference(getURL()));
		}
		
		return w3cList;
	}

}
