package com.qcg.comp.sdk;

import java.util.List;

import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;
import javax.xml.ws.Service;
import javax.xml.ws.WebServiceFeature;
import javax.xml.ws.handler.Handler;

import com.qcg.comp.sdk.exceptions.NotAuthorizedException;
import com.qcg.comp.stubs.bes.management.BESManagement;
import com.qcg.comp.stubs.bes.management.BESManagementPortType;
import com.qcg.comp.stubs.bes.management.NotAuthorizedFault;
import com.qcg.comp.stubs.bes.management.StartAcceptingNewActivitiesType;
import com.qcg.comp.stubs.bes.management.StopAcceptingNewActivitiesType;
import com.qcg.core.sdk.QCGCoreService;
import com.qcg.core.sdk.security.AuthenticationModule;

public class QCGManagement extends QCGCoreService{

	public QCGManagement(String hostname, int port,
			AuthenticationModule transportLayer,
			AuthenticationModule messageLayer, boolean useWSA)
			throws JAXBException {
		super(hostname, port, transportLayer, messageLayer, useWSA);
	}

	public QCGManagement(String hostname, int port,
			AuthenticationModule transportLayer,
			AuthenticationModule messageLayer) throws JAXBException {
		super(hostname, port, transportLayer, messageLayer);
	}

	public QCGManagement(String hostname, int port, AuthenticationModule am)
			throws JAXBException {
		super(hostname, port, am);
	}

	protected BESManagementPortType BESmanagement;
	public static final String BES_MANAGEMENT_NS = "http://schemas.ggf.org/bes/2006/08/bes-management";

	public QCGManagement(String hostname, int port, AuthenticationModule am,
			boolean useWSA) throws JAXBException{
		super(hostname, port, am, useWSA);
		
		BESmanagement = (BESManagementPortType)stubs;
	}
	
	@Override
	protected Service getService(){

		return new BESManagement(QCGManagement.class.getResource("/schemas/bes-management.wsdl"), new QName(BES_MANAGEMENT_NS, "BESManagement"));
	}
	
	@Override
	@SuppressWarnings("all")
	protected List<Handler> getHandlersChain() {
		return null;
	}

	@Override
	protected Object getPort(Service service, WebServiceFeature... features) {
		return ((BESManagement)service).getBESManagement(features);
	}
	
	
	public void startAcceptingNewActivites() throws NotAuthorizedException {
		StartAcceptingNewActivitiesType parameters = new StartAcceptingNewActivitiesType();
		try {
			BESmanagement.startAcceptingNewActivities(parameters);
		} catch (NotAuthorizedFault e) {
			throw new NotAuthorizedException(e);
		}
	
	}
	
	public void stopAcceptingNewActivites() throws NotAuthorizedException {
		StopAcceptingNewActivitiesType parameters = new StopAcceptingNewActivitiesType();
		try {
			BESmanagement.stopAcceptingNewActivities(parameters);
		} catch (NotAuthorizedFault e) {
			throw new NotAuthorizedException(e);
		}
	}
}
