package com.qcg.comp.sdk.ardl;

import java.io.Reader;
import java.io.Serializable;
import java.io.StringReader;
import java.io.StringWriter;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.Duration;

import com.qcg.comp.stubs.ares.ardl.ARDLCandidateHostsType;
import com.qcg.comp.stubs.ares.ardl.ARDLFileSystemType;
import com.qcg.comp.stubs.ares.ardl.ARDLNativeResourceType;
import com.qcg.comp.stubs.ares.ardl.ARDLReservationDefinitionType;
import com.qcg.comp.stubs.ares.ardl.ARDLReservationDescriptionType;
import com.qcg.comp.stubs.ares.ardl.ARDLReservationIdentificationType;
import com.qcg.comp.stubs.ares.ardl.ARDLResourcesType;
import com.qcg.comp.stubs.ares.ardl.ARDLTimeWindowType;
import com.qcg.comp.stubs.ares.ardl.ObjectFactory;
import com.qcg.comp.stubs.jsdl.CPUArchitectureType;
import com.qcg.comp.stubs.jsdl.FileSystemTypeEnumeration;
import com.qcg.comp.stubs.jsdl.OperatingSystemType;
import com.qcg.comp.stubs.jsdl.OperatingSystemTypeEnumeration;
import com.qcg.comp.stubs.jsdl.OperatingSystemTypeType;
import com.qcg.comp.stubs.jsdl.ProcessorArchitectureEnumeration;

public class ARDL implements Serializable, Cloneable {
	
	/**
	 * ver 2.0
	 */
	private static final long serialVersionUID = 210L;

	protected static JAXBContext jaxbCtxt;
	protected static DatatypeFactory datatypeFactory;
	
	protected transient ARDLReservationDefinitionType resDefinition;
	protected String resString;
	
	protected String name;
	protected String description;
	protected List<String> candiateHosts = new ArrayList<String>() ;

	protected boolean openWindow = false;
	protected boolean strictWindow = true;

	protected Date startTime;
	protected Date endTime;
	protected Duration duration; 
	
	protected boolean exclusiveExecution;
	protected OperatingSystemTypeEnumeration operatingSystem;
	protected ProcessorArchitectureEnumeration CPUArchitecture;
	protected Integer totalSlotsCount;
	protected Integer additionalSlotsCount;
	protected Integer individualPhysicalMemory;
	protected Integer individualSwapMemory;
	protected Long individualTmpSpace;
	protected Long individualHomeSpace;
	protected String userName;
	protected String groupName;
	protected List<String> nativeResources =  new ArrayList<String>();
	
	public List<String> getNativeResources() {
		invalidate();
		
		return nativeResources;
	}
	
	protected static synchronized JAXBContext getJAXBContext() throws JAXBException {
		
		if (jaxbCtxt == null) {
			jaxbCtxt = JAXBContext.newInstance(ObjectFactory.class.getPackage().getName());
		}
		
		return jaxbCtxt;
	}
	
	protected static synchronized DatatypeFactory getDatatypeFactory()  {
		
		if (datatypeFactory == null) {
			try {
				datatypeFactory = DatatypeFactory.newInstance();
			} catch (DatatypeConfigurationException e) {
				e.printStackTrace();
			}
		}
		
		return datatypeFactory;
	}
	
	protected void invalidate() {
		resDefinition = null;
		resString = null;
	}
	
	protected void convertToReservationDefinition() {
		if (resDefinition != null)
			return;
		
		resDefinition = new ARDLReservationDefinitionType();
		DatatypeFactory df = getDatatypeFactory();
		
		
		ARDLReservationDescriptionType rd =  new ARDLReservationDescriptionType();
		resDefinition.setReservationDescription(rd);
		
		rd.setTimeWindow(new ARDLTimeWindowType());
		
		//identification
		if (name!= null) {
			rd.setReservationIdentification(new ARDLReservationIdentificationType());
			rd.getReservationIdentification().setReservationName(name);
			
			if (description != null) {
				rd.getReservationIdentification().setDescription(description);
			}
			
		}
		
		//time window
		if (startTime != null) {
			GregorianCalendar gc  = new GregorianCalendar();
			gc.setTime(startTime);
			rd.getTimeWindow().setStartTime(df.newXMLGregorianCalendar(gc));
		}
		
		if (endTime != null) {
			GregorianCalendar gc  = new GregorianCalendar();
			gc.setTime(endTime);
			rd.getTimeWindow().setEndTime(df.newXMLGregorianCalendar(gc));
		}
		
		if (duration != null) {
			rd.getTimeWindow().setDuration(duration);
		}
		
		//resources
		ARDLResourcesType r = new ARDLResourcesType();
		rd.setResources(r);
		
		
		if (exclusiveExecution) {
			r.setExclusiveExecution(exclusiveExecution);
		}
		
		if (operatingSystem != null) {
			r.setOperatingSystem(new OperatingSystemType());
			r.getOperatingSystem().setOperatingSystemType(new OperatingSystemTypeType());
			r.getOperatingSystem().getOperatingSystemType().setOperatingSystemName(operatingSystem);
		}
		
		if (CPUArchitecture != null) {
			r.setCPUArchitecture(new CPUArchitectureType());
			r.getCPUArchitecture().setCPUArchitectureName(CPUArchitecture);
		}

		if (totalSlotsCount != null) {
			r.setReservedSlotsCount(BigInteger.valueOf(totalSlotsCount.longValue()));
		}
		
		if (additionalSlotsCount != null) {
			r.setAdditionalReservedSlotsCount(BigInteger.valueOf(additionalSlotsCount.longValue()));
		}
			
		if (individualPhysicalMemory != null) {
			r.setIndividualPhysicalMemory(BigInteger.valueOf(individualPhysicalMemory.longValue()));
		}
		
		if (individualSwapMemory != null) {
			r.setIndividualSwapMemory(BigInteger.valueOf(individualSwapMemory.longValue()));
		}
		
		if (individualHomeSpace != null) {
			ARDLFileSystemType afs = new ARDLFileSystemType();
			afs.setType(FileSystemTypeEnumeration.NORMAL);
			afs.setValue(BigInteger.valueOf(individualHomeSpace));
			r.getFileSystem().add(afs);
		}
		
		if (individualTmpSpace != null) {
			ARDLFileSystemType afs = new ARDLFileSystemType();
			afs.setType(FileSystemTypeEnumeration.TEMPORARY);
			afs.setValue(BigInteger.valueOf(individualTmpSpace));
			r.getFileSystem().add(afs);
		}
		
		if (userName != null) {
			r.setUserName(userName);
		}
		
		if (groupName != null) {
			r.setGroupName(groupName);
		}
		
		if (candiateHosts.size() != 0) {
			r.setCandidateHosts(new ARDLCandidateHostsType());
			
			for (String host : candiateHosts) 
				r.getCandidateHosts().getHostName().add(host);
		}
		
		if (openWindow)
			rd.getTimeWindow().setOpen(new Boolean(true));
		
		if (!strictWindow)
			rd.getTimeWindow().setStrict(new Boolean(false));
		else 
			rd.getTimeWindow().setStrict(new Boolean(true));

		
		if (nativeResources.size() !=0) {
			for (String nr : nativeResources) {
				ARDLNativeResourceType nrt = new ARDLNativeResourceType();
				String splitted[] = nr.split("=");
				nrt.setName(splitted[0]);

				if (splitted.length == 2) {
					nrt.setValue(splitted[1]);
				} else {
					nrt.setValue(null);
				}
				
				r.getNativeResource().add(nrt);
			}
		}
		
	}
	
	protected void convertFromReservationDefinition() {
		ARDLReservationDescriptionType rd  = resDefinition.getReservationDescription();
		
		//identification
		if (rd.getReservationIdentification() != null) {
			name = rd.getReservationIdentification().getReservationName();
			description = (rd.getReservationIdentification().getDescription() != null ? rd.getReservationIdentification().getDescription() : null);
		}
		
		//time window
		
		if (rd.getTimeWindow().getStartTime() != null)
			startTime = rd.getTimeWindow().getStartTime().toGregorianCalendar().getTime();
		
		if (rd.getTimeWindow().getEndTime() != null)
			endTime = rd.getTimeWindow().getEndTime().toGregorianCalendar().getTime();
		
		duration = rd.getTimeWindow().getDuration();
		
		//resources;
		ARDLResourcesType r = rd.getResources();
		
		
		exclusiveExecution = r.isExclusiveExecution() == null ? false : r.isExclusiveExecution();
		
		operatingSystem = r.getOperatingSystem() == null || 
			r.getOperatingSystem().getOperatingSystemType() == null ? null : r.getOperatingSystem().getOperatingSystemType().getOperatingSystemName();
		CPUArchitecture = r.getCPUArchitecture() == null ? null : r.getCPUArchitecture().getCPUArchitectureName();
		
		totalSlotsCount = r.getReservedSlotsCount() == null ? null : r.getReservedSlotsCount().intValue();
		additionalSlotsCount = r.getReservedSlotsCount() == null ? null : r.getReservedSlotsCount().intValue();
		
		individualPhysicalMemory = r.getIndividualPhysicalMemory() == null ? null : r.getIndividualPhysicalMemory().intValue();
		individualSwapMemory = r.getIndividualPhysicalMemory() == null ? null : r.getIndividualPhysicalMemory().intValue();
		
		for (int i = 0; i < r.getFileSystem().size(); i++) {
			ARDLFileSystemType fst = r.getFileSystem().get(i);
			
			if (fst.getType().equals(FileSystemTypeEnumeration.NORMAL)) {
				individualHomeSpace = fst.getValue().longValue();
			} else if (fst.getType().equals(FileSystemTypeEnumeration.TEMPORARY)) {
				individualTmpSpace = fst.getValue().longValue();
			} else {
				throw new IllegalArgumentException("Unsupported file system type: " + fst.getType().value());
			}
			
		}
		
		
		userName = r.getUserName();
		groupName = r.getGroupName();
		
		if (r.getCandidateHosts() !=  null) {
			for (String hostname : r.getCandidateHosts().getHostName()) {
				candiateHosts.add(hostname);
			}
		}
		
		openWindow = rd.getTimeWindow().isOpen();
		strictWindow = rd.getTimeWindow().isStrict();
		
		for (int i=0; i < rd.getResources().getNativeResource().size(); i++ ) {
			String name = rd.getResources().getNativeResource().get(i).getName();
			String value = rd.getResources().getNativeResource().get(i).getValue();
			String cat = name;
			
			if (value!= null && (!"".equals(value))) {
				cat+=("=" + name);
			}
			nativeResources.add(cat);
		}
	}
	
	public ARDL(Date endTime) {
		this.endTime = endTime;
	}
	
	public ARDL(Date startTime, Date endTime) {
		this.startTime = startTime;
		this.endTime = endTime;
	}
	
	public ARDL(Duration duration) {
		this.duration = duration;
	}
	
	public ARDL(int duration) {
		this.duration = getDatatypeFactory().newDuration(duration * 1000);
	}
	
	public ARDL(ARDLReservationDefinitionType ar) {
		this.resDefinition = ar;
		
		convertFromReservationDefinition();
	}
	
	@SuppressWarnings("unchecked")
	public ARDL(Reader reader) throws JAXBException {
		Unmarshaller um = getJAXBContext().createUnmarshaller();
		resDefinition = ((JAXBElement<ARDLReservationDefinitionType>) um.unmarshal(reader)).getValue();
		
		convertFromReservationDefinition();
	}
	
	public ARDLReservationDefinitionType getReservationDefinition() {
		convertToReservationDefinition();
		
		return resDefinition;
	}


	public void setReservationDefinition(ARDLReservationDefinitionType resDefinition) {
		invalidate();
		
		this.resDefinition = resDefinition;
	}


	public String getName() {
		return name;
	}

	public void setName(String name) {
		invalidate();
		
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		invalidate();
		
		this.description = description;
	}

	public Date getStartTime() {
		return startTime;
	}

	public void setStartTime(Date startTime) {
		invalidate();
		
		this.startTime = startTime;
	}

	public Date getEndTime() {
		return endTime;
	}

	public void setEndTime(Date endTime) {
		invalidate();
		
		this.endTime = endTime;
	}

	public Duration getDuration() {
		return duration;
	}

	public void setDuration(Duration duration) {
		invalidate();
		
		this.duration = duration;
	}

	public boolean isExclusiveExecution() {
		return exclusiveExecution;
	}

	public void setExclusiveExecution(boolean exclusiveExecution) {
		invalidate();

		this.exclusiveExecution = exclusiveExecution;
	}

	public OperatingSystemTypeEnumeration getOperatingSystem() {
		return operatingSystem;
	}

	public void setOperatingSystem(OperatingSystemTypeEnumeration operatingSystem) {
		invalidate();

		this.operatingSystem = operatingSystem;
	}

	public ProcessorArchitectureEnumeration getCPUArchitecture() {
		return CPUArchitecture;
	}

	public void setCPUArchitecture(ProcessorArchitectureEnumeration architecture) {
		invalidate();

		CPUArchitecture = architecture;
	}

	public Integer getTotalSlotsCount() {
		return totalSlotsCount;
	}

	public void setTotalSlotsCount(Integer totalSlotsCount) {
		invalidate();

		this.totalSlotsCount = totalSlotsCount;
	}

	public Integer getIndividualPhysicalMemory() {
		return individualPhysicalMemory;
	}

	public void setIndividualPhysicalMemory(Integer individualPhysicalMemory) {
		invalidate();

		this.individualPhysicalMemory = individualPhysicalMemory;
	}

	public Integer getIndividualSwapMemory() {
		return individualSwapMemory;
	}

	public void setIndividualSwapMemory(Integer individualSwapMemory) {
		invalidate();

		this.individualSwapMemory = individualSwapMemory;
	}

	public Long getIndividualHomeSpace() {
		return individualHomeSpace;
	}

	public void setIndividualHomeSpace(Long individualHomeSpace) {
		invalidate();

		this.individualHomeSpace = individualHomeSpace;
	}

	public String getUserName() {
		return userName;
	}

	public void setUserName(String userName) {
		invalidate();

		this.userName = userName;
	}

	public String getGroupName() {
		return groupName;
	}

	public void setGroupName(String groupName) {
		invalidate();

		this.groupName = groupName;
	}

	public List<String> getCandiateHosts() {
		invalidate();
		
		return candiateHosts;
	}
	
	public Integer getAdditionalSlotsCount() {
		return additionalSlotsCount;
	}

	public void setAdditionalSlotsCount(Integer additionalSlotsCount) {
		invalidate();
		
		this.additionalSlotsCount = additionalSlotsCount;
	}
	
	public boolean isOpenWindow() {
		return openWindow;
	}

	public void setOpenWindow(boolean openWindow) {
		invalidate();
		
		this.openWindow = openWindow;
	}
	
	public boolean isStrictWindow() {
		return strictWindow;
	}

	public void setStrictWindow(boolean strictWindow) {
		invalidate();
		this.strictWindow = strictWindow;
	}

	@Override
	public String toString() {
		try {
			if (resString == null) {
				convertToReservationDefinition();
				Marshaller m = getJAXBContext().createMarshaller();
				m.setProperty("jaxb.formatted.output",new Boolean(true)); 
				StringWriter sw  = new StringWriter(512);
				m.marshal((new ObjectFactory()).createReservationDefinition(resDefinition), sw);
				resString = sw.getBuffer().toString();
			}
		} catch (Exception ex) {
			ex.printStackTrace();
			resString = "Unable to marshal JSDL:" + ex.getMessage();
		}
		
		return resString;
	}
	
	@Override
	public Object clone() throws CloneNotSupportedException {
		String ardlStr = this.toString();
		try {
			return new ARDL(new StringReader(ardlStr));
		} catch (JAXBException e) {
			e.printStackTrace();
			return null;
		}
	}

	
}
