package com.qcg.comp.sdk.jsdl;

import java.io.Serializable;

import com.qcg.comp.stubs.hpcp.fs.CredentialType;
import com.qcg.comp.stubs.jsdl.CreationFlagEnumeration;
import com.qcg.comp.stubs.jsdl.DataStagingType;
import com.qcg.comp.stubs.jsdl.SourceTargetType;



public class JSDLDataStaging implements Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = 200L;
	
	protected String fileName;
	protected String sourceURL;
	protected String targetURL;
	protected boolean deleteOnTermination;
	protected CreationFlagEnumeration creationFlag;
	protected transient CredentialType credentials;
	protected String fileMode;
	protected Integer offset;
	
	public JSDLDataStaging(String fileName, String sourceURL, String targetURL) {
		this(CreationFlagEnumeration.OVERWRITE, false, fileName, sourceURL, targetURL);
	}
	
	public JSDLDataStaging(CreationFlagEnumeration creationFlag,
			boolean deleteOnTermination, String fileName, String sourceURL,
			String targetURL) {
		this.creationFlag = creationFlag;
		this.deleteOnTermination = deleteOnTermination;
		this.fileName = fileName;
		this.sourceURL = sourceURL;
		this.targetURL = targetURL;
	}
	
	public JSDLDataStaging(DataStagingType ds) {
		this.creationFlag = ds.getCreationFlag();
		this.deleteOnTermination = ds.isDeleteOnTermination() == null ? false : ds.isDeleteOnTermination();
		this.fileName =  ds.getFileName();
		if (ds.getSource() != null)
			this.sourceURL = ds.getSource().getURI();
		
		if (ds.getTarget() != null)
			this.targetURL = ds.getTarget().getURI();
		
		this.credentials = ds.getCredential();
		
		this.fileMode = ds.getFileMode();
		this.offset = ds.getOffset();
	}

	public String getFileName() {
		return fileName;
	}
	public void setFileName(String fileName) {
		this.fileName = fileName;
	}
	public String getSourceURL() {
		return sourceURL;
	}
	public void setSourceURL(String sourceURL) {
		this.sourceURL = sourceURL;
	}
	public String getTargetURL() {
		return targetURL;
	}
	public void setTargetURL(String targetURL) {
		this.targetURL = targetURL;
	}
	public boolean isDeleteOnTermination() {
		return deleteOnTermination;
	}
	public void setDeleteOnTermination(boolean deleteOnTermination) {
		this.deleteOnTermination = deleteOnTermination;
	}
	public CreationFlagEnumeration getCreationFlag() {
		return creationFlag;
	}
	public void setCreationFlag(CreationFlagEnumeration creationFlag) {
		this.creationFlag = creationFlag;
	}
	
	public CredentialType getCredentials() {
		return credentials;
	}

	public void setCredentials(CredentialType credentials) {
		this.credentials = credentials;
	}
	
	public String getFileMode() {
	    return fileMode;
	}

	public void setFileMode(String fileMode) {
	    this.fileMode = fileMode;
	}

	public Integer getOffset() {
	    return offset;
	}

	public void setOffset(Integer offset) {
	    this.offset = offset;
	}	

	public DataStagingType getDataStagingElement() {
		DataStagingType ds = new DataStagingType();
		
		ds.setDeleteOnTermination(deleteOnTermination);
		ds.setCreationFlag(creationFlag);
		ds.setFileName(fileName);
		
		if (sourceURL != null) {
			ds.setSource(new SourceTargetType());
			ds.getSource().setURI(sourceURL);
		}
		
		if (targetURL != null) {
			ds.setTarget(new SourceTargetType());
			ds.getTarget().setURI(targetURL);
		}
		
		ds.setCredential(credentials);
		
		ds.setFileMode(fileMode);
		ds.setOffset(offset);
		
		return ds;
	}

}
