package com.qcg.comp.tests;

import static org.junit.Assert.*;

import java.io.File;
import java.util.ArrayList;

import org.junit.Before;
import org.junit.Test;

import com.qcg.comp.sdk.QCGStaging;
import com.qcg.comp.sdk.jsdl.JSDL;
import com.qcg.comp.sdk.jsdl.JSDLDataStaging;
import com.qcg.comp.sdk.types.FileInfo;
import com.qcg.core.sdk.security.AuthenticationModule;
import com.qcg.core.sdk.security.anonymous.AnonymousAuthentication;

public class ListDirectoryStagingTest {

    protected QCGStaging staging;

    public static final String baseDir = "/tmp";       

    @Before
    public void setUp() throws Exception {
	String hostname = "localhost";
	int port = 19000;
	boolean useWSA = true;
	AuthenticationModule am = new AnonymousAuthentication();

	staging = new QCGStaging(hostname, port, am, useWSA, null);	
    }

    @Test
    public void listFiles() throws Throwable {
	JSDL jsdl = staging.listDirectory(baseDir, null);		
	File dir = new File(baseDir);
	
	ArrayList<String> local = new ArrayList<String>();	
	
	for(String f : dir.list())
	    local.add(f);
	
	for(JSDLDataStaging ds : jsdl.getDataStaging())
	    local.remove(ds.getFileName());
	
	assertTrue(local.isEmpty());
    }
    
    @Test
    public void statFile() throws Throwable {
	ArrayList<String> files = new ArrayList<String>();
	files.add("/etc/passwd");
	FileInfo[] file = staging.statFile(files, null);
	assertEquals(1, file.length);
	File f=new File("/etc/passwd");	
	assertEquals(f.length(), file[0].getSize());
	assertEquals("root", file[0].getOwner());
    }
    
    @Test
    public void delete() throws Throwable {
	File f=File.createTempFile("staging", "delete");
	assertTrue(f.exists());
	staging.delete(f.getAbsolutePath(), null);
	assertFalse(f.exists());
    }

    @Test
    public void chmod() throws Throwable {
	File f = File.createTempFile("staing", "chmod");
	f.deleteOnExit();
	f.setWritable(true, true);
	f.setReadable(false, false);
	assertFalse(f.canRead());
	assertTrue(f.canWrite());
	staging.chmod(f.getAbsolutePath(), "0444", null);
	assertTrue(f.canRead());
	assertFalse(f.canWrite());
    }

    @Test
    public void mkdir() throws Throwable {
	String path = "/tmp/staging_test";
	staging.mkdir(path, null, null);
	File f = new File(path);
	assertTrue(f.exists());
	assertTrue(f.isDirectory());
	f.delete();
    }

    @Test
    public void mkdir_with_perms() throws Throwable {
	String path = "/tmp/staging_test";
	staging.mkdir(path, "555", null);
	File f = new File(path);
	assertTrue(f.exists());
	assertTrue(f.isDirectory());
	assertTrue(f.canRead());
	assertFalse(f.canWrite());
	f.delete();
    }
}
