package com.qcg.comp.tests;



import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.Inet4Address;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import junit.framework.TestCase;

import com.qcg.comp.sdk.ARES;
import com.qcg.comp.sdk.QCGFactory;
import com.qcg.comp.sdk.QCGStaging;
import com.qcg.comp.sdk.exceptions.NotAuthorizedException;
import com.qcg.comp.sdk.exceptions.UnknownActivityIdentifierException;
import com.qcg.comp.sdk.jsdl.JSDL;
import com.qcg.comp.sdk.types.ActivityEndpointReference;
import com.qcg.comp.sdk.types.ReservationEndpointReference;
import com.qcg.comp.sdk.types.QCGActivityStatus;
import com.qcg.comp.stubs.ares.ARESReservationStatusType;
import com.qcg.comp.stubs.ares.ARESReservedHostType;
import com.qcg.core.sdk.attachments.SimpleFileStagingHandler;
import com.qcg.core.sdk.security.AuthenticationModule;
import com.qcg.core.sdk.security.anonymous.AnonymousAuthentication;
import com.qcg.core.sdk.utils.IOUtils;

/**
 * Unit test for simple App.
 */
public class QCGCompConfig 
    extends TestCase
{
	public static final String TESTSUITE_PFX = "testsuite.";
	
	protected QCGFactory factory;
	protected ARES ares;
	protected QCGStaging staging;
	protected String username1;
	protected String username2;
	protected Properties config = new Properties();
	protected List<String> disabledTests = new ArrayList<String>();
	protected ActivityEndpointReference jobEpr1;
	protected ActivityEndpointReference jobEpr2;
	protected ReservationEndpointReference resEpr1;
	protected ReservationEndpointReference resEpr2;
	protected JSDL jsdl;
	protected ExecutorService pool = Executors.newCachedThreadPool();
	protected String localHostname; 

	@Override
	protected void setUp() throws Exception {
		String confFileName = System.getProperty(TESTSUITE_PFX + "config", "/default.properties");
		config.load(new FileInputStream(getRes(confFileName)));
		
		localHostname = Inet4Address.getLocalHost().getHostName();
		
		String hostname = config.getProperty(TESTSUITE_PFX + "hostname");
		int port = Integer.parseInt(config.getProperty(TESTSUITE_PFX + "port"));
		boolean useWSA = Boolean.parseBoolean(config.getProperty(TESTSUITE_PFX + "useWSA"));
		
		System.err.println("Using QCGComp: " + hostname + ":" + port);
		
		AuthenticationModule am = null;
		String authMethod  = config.getProperty(TESTSUITE_PFX + "authentication"); 
		if (authMethod.equalsIgnoreCase("anonymous")) {
			am = new AnonymousAuthentication();
		} else if (authMethod.equals("gsi")) {
			//am = GSIAuthentication("/O=Grid/OU=QosCosGrid/CN=opendsp/node2.qoscosgrid.man.poznan.pl");
		} else {
			throw new IllegalArgumentException("Unknown authentication method");
		}
		
		boolean soapDump = Boolean.parseBoolean(config.getProperty(TESTSUITE_PFX + "soapdump", "false"));
		System.setProperty("com.sun.xml.ws.transport.http.client.HttpTransportPipe.dump", soapDump +"");
		
		username1 = config.getProperty(TESTSUITE_PFX + "user1.name");
		username2 = config.getProperty(TESTSUITE_PFX + "user2.name");
		
		factory = new QCGFactory(hostname, port, am,  useWSA);
		ares =  new ARES(hostname, port, am,  useWSA);
		staging = new QCGStaging(hostname, port, am, useWSA, new SimpleFileStagingHandler());
		
		String dts[] = config.getProperty(TESTSUITE_PFX + "disabledTests").split("\\s*,\\s*");
		
		for (String dt : dts) {
			disabledTests.add(dt);
		}

	}
	
	
	@Override
	protected void tearDown() throws Exception {
		factory = null;
		ares =  null;
		staging = null;
	}


	public void waitForTermination(ActivityEndpointReference epr) throws NotAuthorizedException, UnknownActivityIdentifierException, InterruptedException  {
		QCGActivityStatus stat = null;
		while (!(stat = factory.getActivityStatus(epr)).isFinalState()) {
			System.err.println(stat);
			Thread.sleep(2000);
		}

		System.err.println(stat);
	}
	
	public void waitForState(ActivityEndpointReference epr, String state) throws NotAuthorizedException, UnknownActivityIdentifierException, InterruptedException {
		QCGActivityStatus stat = null;
		while (!(stat = factory.getActivityStatus(epr)).getState().equals(state)) {
			System.err.println(stat);
			Thread.sleep(2000);
		}
		
		System.err.println(stat);
	}

	
	public boolean isDisabled(){
		String className[] = this.getClass().getName().split("\\.");
		boolean res = false;
		
		for (String dt :  disabledTests ) {
			//System.err.println("comparing : " + dt + " with " + className[className.length - 1]);
			 res =  res || className[className.length - 1].matches(dt);
		}
		
		if (res) {
			System.err.println("TEST DISABLED: " + className[className.length -1]);
		}
		
		return res;
	}
	
	protected String getRes(String name){
		if ((new File(name)).exists())
			return name;
		
		return QCGCompConfig.class.getResource(name).getPath();
	}
	
	
	public static void assertFileEquals(File f1, File f2) throws IOException {
		assertEquals(IOUtils.fileToString(f1), IOUtils.fileToString(f2));
	}


	@Override
	protected void runTest() throws Throwable {
		if (!isDisabled())
			super.runTest();
	}
	
	public static void printReservationStatus(ARESReservationStatusType rt) {
		System.err.println("reserved hosts");
		
		if (rt.getReservedHost() != null) {
			for (ARESReservedHostType rh : rt.getReservedHost()) {
				System.err.println("[host=" + rh.getHostName() 
						+ ",totalCpu=" +  rh.getTotalSlotsCount() 
						+ ",reservedCpu=" + rh.getReservedSlotsCount()
						+ ",usedCpu=" + rh.getUsedSlotsCount() + "]");
			}
		}
	}
	
}
