package com.qcg.comp.tests.authentication;

import javax.xml.bind.JAXBException;

import junit.framework.TestCase;

import com.qcg.comp.sdk.QCGFactory;
import com.qcg.comp.sdk.exceptions.CantApplyOperationToCurrentStateException;
import com.qcg.comp.sdk.exceptions.InvalidRequestMessageException;
import com.qcg.comp.sdk.exceptions.NotAuthorizedException;
import com.qcg.comp.sdk.exceptions.UnknownActivityIdentifierException;
import com.qcg.comp.sdk.exceptions.UnsupportedFeatureException;
import com.qcg.comp.sdk.jsdl.JSDL;
import com.qcg.comp.sdk.types.ActivityEndpointReference;
import com.qcg.core.sdk.security.anonymous.AnonymousAuthentication;

public class AnonymousAuthenticationTest extends TestCase {
	static final String QCG_COMP_HOST = "grass1.man.poznan.pl";
	static final int QCG_COMP_PORT_1 = 19001;
	static final int QCG_COMP_PORT_2 = 19003;
	static final String USERNAME = "testsuite";
	
	public void testFixedMapping() throws InvalidRequestMessageException, NotAuthorizedException, UnsupportedFeatureException, JAXBException, CantApplyOperationToCurrentStateException, UnknownActivityIdentifierException {

		
		QCGFactory executionFactory = new QCGFactory(QCG_COMP_HOST, QCG_COMP_PORT_1, new AnonymousAuthentication());

		JSDL jsdl = new JSDL("DateExample");
		jsdl.setExecutable("/bin/date");
		
		ActivityEndpointReference jobEpr = executionFactory.createActivity(jsdl);
		
		System.err.println("Submitted activity: " + jobEpr);
		
		assertEquals(USERNAME, executionFactory.getActivityDocuments(jobEpr).getJsdl().getUserName());
		
		executionFactory.terminateActivity(jobEpr);
	}
	

	public void testWildcardMapping() throws InvalidRequestMessageException, NotAuthorizedException, UnsupportedFeatureException, JAXBException, CantApplyOperationToCurrentStateException, UnknownActivityIdentifierException {

		
		QCGFactory executionFactory = new QCGFactory(QCG_COMP_HOST, QCG_COMP_PORT_2, new AnonymousAuthentication());

		JSDL jsdl = new JSDL("DateExample");
		jsdl.setExecutable("/bin/date");
		
		jsdl.setUserName(USERNAME);
		
		ActivityEndpointReference jobEpr = executionFactory.createActivity(jsdl);
		
		System.err.println("Submitted activity: " + jobEpr);
		
		assertEquals(USERNAME, executionFactory.getActivityDocuments(jobEpr).getJsdl().getUserName());
		
		executionFactory.terminateActivity(jobEpr);
	}
}
