/* $Id: reservation_module.h 2 2009-10-05 10:09:52Z mamonski $ */

/*
 * Copyright 2006-2009 QCG Project Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __SMC_RESERVATION_MODULE_H
#define __SMC_RESERVATION_MODULE_H

#include <qcg-core/value.h>
#include <qcg-core/modules/modules.h>

/* These are mapped directly to values in faults.c */
typedef enum {
	RESERVATION_OK = 0,
	RESERVATION_INTERNAL_FAULT = 1,
	RESERVATION_NOT_AUTHORIZED_FAULT = 300,
	RESERVATION_UNSUPPORTED_FEATURE_FAULT,
	RESERVATION_INVALID_REQUEST_MESSAGE_FAULT,
	RESERVATION_UNKNOWN_RESERVATION_IDENTIFIER_FAULT,
	RESERVATION_NO_SLOTS_AVAILABLE_FAULT,
	RESERVATION_NO_MATCHING_RESOURCES_FAULT
} reservation_ret_code_t;

typedef sm_value_t *(*reservation_create_function)(sm_module_t this, sm_value_t *ardl);
typedef sm_value_t *(*reservation_terminate_function)(sm_module_t this, sm_value_t *identifier);

/*
 * result[1].varray	array of strings in following format: hostname:total_slots:reserved_slots:slots_used_by_reservation
 */
typedef sm_value_t *(*reservation_get_status_function)(sm_module_t this, sm_value_t *identifier);
typedef sm_value_t *(*reservation_get_active_function)(sm_module_t this);

typedef struct reservation_module_s {
        struct sm_module_s header;

        reservation_create_function create;
        reservation_terminate_function terminate;
	reservation_get_status_function get_status;
	reservation_get_active_function get_active;
} *reservation_module_t;

#define RESERVATION_MODULE		struct reservation_module_s __module_entry__
#define RESERVATION_MODULE_ENTRY_SIZE	sizeof(struct reservation_module_s)

#define RESERVATION_MODULE_API		SM_BIT(SM_API_CUSTOM + 1)

#endif /* __SMC_RESERVATION_MODULE_H */
