/* $Id: submission_module.h 324 2011-01-07 14:53:02Z mamonski $ */

/*
 * Copyright 2006-2009 QCG Project Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __SMC_SUBMISSION_MODULE_H
#define __SMC_SUBMISSION_MODULE_H

#include <qcg-core/value.h>
#include <qcg-core/modules/modules.h>

/* These are mapped directly to values in faults.c */
typedef enum {
	SUBMISSION_OK = 0,
	SUBMISSION_INTERNAL_FAULT = 1,
	SUBMISSION_NOT_AUTHORIZED_FAULT = 100,
	SUBMISSION_NOT_ACCEPTING_NEW_ACTIVITIES_FAULT,
	SUBMISSION_UNSUPPORTED_FEATURE_FAULT,
	SUBMISSION_UNKNOWN_ACTIVITY_IDENTIFIER_FAULT,
	SUBMISSION_INVALID_REQUEST_MESSAGE_FAULT
} submission_ret_code_t;

typedef enum {
	SUBMISSION_DRMS_UNKNOWN,
	SUBMISSION_DRMS_GE,
	SUBMISSION_DRMS_LSF,
	SUBMISSION_DRMS_TORQUE,
	SUBMISSION_DRMS_PBSPRO,
	SUBMISSION_DRMS_XGRID
} submission_drms_t;

typedef enum {
	SUBMISSION_HELD = 2,
	SUBMISSION_QUEUED = 3,
	SUBMISSION_RUNNING = 4,
	SUBMISSION_SUSPENDED = 5
} submission_status_t;

typedef enum {
	SUBMISSION_FINISHED = 7,
	SUBMISSION_FAILED = 8
} submission_wait_result_t;

typedef sm_value_t *(*submission_submit_function)(sm_module_t this, sm_value_t *params);
typedef sm_value_t *(*submission_terminate_function)(sm_module_t this, sm_value_t *identifiers);
typedef sm_value_t *(*submission_get_statuses_function)(sm_module_t this, sm_value_t *identifiers);
/**
 IN:
none 
 OUT:
result[0] : ret code (RETCODE)
result[1] : job status (INT_32) - @see submission_wait_result_t
result[2] : job id
result[3] : exit status (INT_32) | core dumped (BOOLEAN)
result[4] : signal name (STRING) | NONE
result[5] : rusage (ARRAY of STRINGs) | NONE
**/
typedef sm_value_t *(*submission_wait_function)(sm_module_t this);
typedef sm_value_t *(*submission_get_drms_function)(sm_module_t this);

typedef struct submission_module_s {
        struct sm_module_s header;

        submission_submit_function submit;
	submission_terminate_function terminate;
	submission_get_statuses_function get_statuses;
	submission_wait_function wait;
	submission_get_drms_function get_drms;
} *submission_module_t;

#define SUBMISSION_MODULE		struct submission_module_s __module_entry__
#define SUBMISSION_MODULE_ENTRY_SIZE	sizeof(struct submission_module_s)

#define SUBMISSION_MODULE_API		SM_BIT(SM_API_CUSTOM)

#endif /* __SMC_SUBMISSION_MODULE_H */
