/* $Id: faults.h 126 2010-06-11 09:24:17Z mamonski $ */

/*
 * Copyright 2006-2009 QCG Project Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __SMC_FAULTS_H
#define __SMC_FAULTS_H

#include <sys/types.h>

#include <qcg-core/transport.h>
#include <qcg-core/value.h>

typedef enum {
	bes_factory_OK = 0,
	bes_factory_InternalFault = 1,
	bes_factory_NotAuthorizedFault = 100,
	bes_factory_NotAcceptingNewActivitiesFault = 101,
	bes_factory_UnsupportedFeatureFault = 102,
	bes_factory_UnknownActivityIdentifierFault = 103,
	bes_factory_InvalidRequestMessageFault = 104,
	bes_factory_CantApplyOperationToCurrentStateFault = 105
} bes_factory_fault_t;

typedef enum {
	bes_management_OK = 0,
	bes_management_InternalFault = 1,
	bes_management_NotAuthorizedFault = 200
} bes_management_fault_t;

typedef enum {
	qcg_comp_ares_OK = 0,
	qcg_comp_ares_InternalFault = 1,
	qcg_comp_ares_NotAuthorizedFault = 300,
	qcg_comp_ares_UnsupportedFeatureFault = 301,
	qcg_comp_ares_InvalidRequestMessageFault = 302,
	qcg_comp_ares_UnknownReservationIdentifierFault = 303,
	qcg_comp_ares_NoSlotsAvailableFault = 304,
	qcg_comp_ares_NoMatchingResourcesFault = 305
} qcg_comp_ares_fault_t;

typedef enum {
	qcg_comp_staging_OK = 0,
	qcg_comp_staging_InternalFault = 1,
	qcg_comp_staging_DeleteOnTerminationFault = 400,
	qcg_comp_staging_FileNotFoundFault = 401,
	qcg_comp_staging_StagingFault = 402,
	qcg_comp_staging_NotAuthorizedFault  = 403
} qcg_comp_staging_fault_t;

typedef enum
{
	RSyncNoError = 0,
	RSyncInternalError = 1,
	RSyncIOError = 500,
	RSyncInvalidPathError = 501,
	RSyncNoMemoryError = 502,
	RSyncAttachmentNotFound = 503
} qcg_comp_rsync_fault_t;

const char *smc_get_fault_msg(sm_transport_t, sm_transport_fault_t);
sm_transport_fault_class_t smc_get_fault_class(sm_transport_t, sm_transport_fault_t);
void *smc_create_fault_detail(sm_transport_t, sm_transport_fault_t);
const char *smc_fault_code2s(sm_transport_t, sm_transport_fault_t);
void smc_free_fault_data(sm_transport_t, void **);

#define SMC_FAULT_INIT \
	do { \
		fault = &faultstruct; \
		faultstruct.type = SM_GENERAL_FAULT; \
		faultstruct.general_fault.class = SM_RECEIVER_FAULT; \
		faultstruct.general_fault.msg = "Internal fault"; \
	} while (0);

#define THROW_INTERNAL_FAULT(message) \
	do { \
		fault->type = SM_GENERAL_FAULT; \
		fault->general_fault.class = SM_RECEIVER_FAULT; \
		fault->general_fault.msg = message; \
		goto fault; \
	} while (0)

#define THROW_APP_FAULT(ecode, message) \
	do { \
		memset(fault, 0, sizeof(*fault)); \
		fault->type = SM_APPLICATION_FAULT; \
		fault->application_fault.code = ecode; \
		fault->application_fault.data = (message ? sm_value_create(1, SM_VALUE_STRING, message) : NULL); \
		goto fault; \
	} while (0)

#define THROW_APP_FAULT2STR(ecode, message, detail) \
	do { \
		memset(fault, 0, sizeof(*fault)); \
		fault->type = SM_APPLICATION_FAULT; \
		fault->application_fault.code = ecode; \
		fault->application_fault.data = sm_value_create(2, SM_VALUE_STRING, message, SM_VALUE_STRING, detail); \
		goto fault; \
	} while (0)

#define THROW_APP_FAULT2INT(ecode, message, detail) \
	do { \
		memset(fault, 0, sizeof(*fault)); \
		fault->type = SM_APPLICATION_FAULT; \
		fault->application_fault.code = ecode; \
		fault->application_fault.data = sm_value_create(2, SM_VALUE_STRING, message, SM_VALUE_INT32, detail); \
		goto fault; \
	} while (0)

#define THROW_APP_FAULT_FROM_VAL(value) \
	do { \
		size_t __count; \
		memset(fault, 0, sizeof(*fault)); \
		fault->type = SM_APPLICATION_FAULT; \
		fault->application_fault.code = SM_VALUE_RET_CODE_C(value[0]); \
		fault->application_fault.data = sm_value_create(1, SM_VALUE_STRING, SM_VALUE_RET_CODE_STRING(value[0])); \
		__count = sm_value_count(value) - 1; \
		if (__count) { \
			size_t __i; \
			for (__i = 1 ; __i <= __count; __i++) \
				sm_value_append(fault->application_fault.data, value[__i]->type, value[__i]->vany); \
		} \
		goto fault; \
	} while (0)

#endif /* __SMC_FAULTS_H */
