/* $Id: icomm.h 294 2010-08-30 08:51:28Z smaug $ */

/*
 * Copyright 2006-2009 QCG Project Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __SMC_ICOMM_H
#define __SMC_ICOMM_H

#include <sys/types.h>

#include <qcg-core/sm.h>
#include <qcg-core/value.h>
#include <qcg-core/modules/modules.h>

typedef enum {
	smc_icomm_CreateActivity,
	smc_icomm_GetActivityStatuses,
	smc_icomm_TerminateActivities,
	smc_icomm_CreateReservation,
	smc_icomm_TerminateReservation,
	smc_icomm_GetReservationStatus,
	smc_icomm_GetActiveReservations,
	smc_icomm_StageInCreateContext,
	smc_icomm_StageInOpen,
	smc_icomm_StageInWrite,
	smc_icomm_StageInClose,
	smc_icomm_StageInDestroyContext,
	smc_icomm_StageOutCreateContext,
	smc_icomm_StageOutOpen,
	smc_icomm_StageOutRead,
	smc_icomm_StageOutClose,
	smc_icomm_StageOutDestroyContext,
	smc_icomm_GetFiles,
	smc_icomm_GetSignature,
	smc_icomm_PutDeltaBegin,
	smc_icomm_PutDeltaProcess,
	smc_icomm_GetDeltaBegin,
	smc_icomm_GetDeltaProcess,
	smc_icomm_ListDirectory,
	smc_icomm_StatFile,
	smc_icomm_Delete,
	smc_icomm_ChMod,
	smc_icomm_MkDir,
} smc_icomm_request_t;

sm_status_ret_code_t smc_icomm_setup(sm_module_t this);
void smc_icomm_cleanup(sm_module_t this);

sm_status_ret_code_t smc_icomm_request_CreateActivity(uid_t uid, sm_value_t *params, sm_value_t **result);
sm_status_ret_code_t smc_icomm_request_GetActivityStatuses(uid_t uid, sm_value_t *params, sm_value_t **result);
sm_status_ret_code_t smc_icomm_request_TerminateActivities(uid_t uid, sm_value_t *params, sm_value_t **result);
sm_status_ret_code_t smc_icomm_request_CreateReservation(uid_t uid, sm_value_t *params, sm_value_t **result);
sm_status_ret_code_t smc_icomm_request_TerminateReservation(uid_t uid, sm_value_t *params, sm_value_t **result);
sm_status_ret_code_t smc_icomm_request_GetReservationStatus(uid_t uid, sm_value_t *params, sm_value_t **result);
sm_status_ret_code_t smc_icomm_request_GetActiveReservations(uid_t uid, sm_value_t *empty, sm_value_t **result);
sm_status_ret_code_t smc_icomm_request_StageInCreateContext(uid_t uid, sm_value_t *params, sm_value_t **result, int *context, void **handle);
sm_status_ret_code_t smc_icomm_request_StageInOpen(int context, sm_value_t *params, sm_value_t **result);
sm_status_ret_code_t smc_icomm_request_StageInWrite(int context, sm_value_t *params, sm_value_t **result);
sm_status_ret_code_t smc_icomm_request_StageInClose(int context, sm_value_t *params, sm_value_t **result);
sm_status_ret_code_t smc_icomm_request_StageInDestroyContext(int *context, void **handle, sm_value_t *empty, sm_value_t **result);
sm_status_ret_code_t smc_icomm_request_StageOutCreateContext(uid_t uid, sm_value_t *params, sm_value_t **result, int *context, void **handle);
sm_status_ret_code_t smc_icomm_request_StageOutOpen(int context, sm_value_t *params, sm_value_t **result);
sm_status_ret_code_t smc_icomm_request_StageOutRead(int context, sm_value_t *params, sm_value_t **result);
sm_status_ret_code_t smc_icomm_request_StageOutClose(int context, sm_value_t *params, sm_value_t **result);
sm_status_ret_code_t smc_icomm_request_StageOutDestroyContext(int *context, void **handle, sm_value_t *empty, sm_value_t **result);

sm_status_ret_code_t smc_icomm_begin_receive(int fd, smc_icomm_request_t *result);

sm_status_ret_code_t smc_icomm_receive_CreateActivity(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_receive_GetActivityStatuses(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_receive_TerminateActivities(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_receive_CreateReservation(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_receive_TerminateReservation(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_receive_GetReservationStatus(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_receive_GetActiveReservations(int fd, sm_value_t **empty);
sm_status_ret_code_t smc_icomm_receive_StageInCreateContext(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_receive_StageInOpen(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_receive_StageInWrite(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_receive_StageInClose(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_receive_StageInDestroyContext(int fd, sm_value_t **empty);
sm_status_ret_code_t smc_icomm_request_StageInRelease(int *context, void **handle);
sm_status_ret_code_t smc_icomm_receive_StageOutCreateContext(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_receive_StageOutOpen(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_receive_StageOutRead(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_receive_StageOutClose(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_receive_StageOutDestroyContext(int fd, sm_value_t **empty);
sm_status_ret_code_t smc_icomm_request_StageOutRelease(int *context, void **handle);

sm_status_ret_code_t smc_icomm_respond_CreateActivity(int fd, sm_value_t *result);
sm_status_ret_code_t smc_icomm_respond_GetActivityStatuses(int fd, sm_value_t *result);
sm_status_ret_code_t smc_icomm_respond_TerminateActivities(int fd, sm_value_t *result);
sm_status_ret_code_t smc_icomm_respond_CreateReservation(int fd, sm_value_t *result);
sm_status_ret_code_t smc_icomm_respond_TerminateReservation(int fd, sm_value_t *result);
sm_status_ret_code_t smc_icomm_respond_GetReservationStatus(int fd, sm_value_t *result);
sm_status_ret_code_t smc_icomm_respond_GetActiveReservations(int fd, sm_value_t *result);
sm_status_ret_code_t smc_icomm_respond_StageIn(int fd, sm_value_t *result);
sm_status_ret_code_t smc_icomm_respond_StageOut(int fd, sm_value_t *result);

#define ICOMM_DECL(name) \
		sm_status_ret_code_t smc_icomm_request_##name(uid_t uid, sm_value_t *params, sm_value_t **result);\
		sm_status_ret_code_t smc_icomm_respond_##name(int fd, sm_value_t *result);\
		sm_status_ret_code_t smc_icomm_receive_##name(int fd, sm_value_t **params);

/*TODO: make it coherent */

ICOMM_DECL(GetFiles)
ICOMM_DECL(GetSignature)
ICOMM_DECL(PutDelta)
sm_status_ret_code_t smc_icomm_request_GetDeltaBegin(uid_t uid, sm_value_t *params, sm_value_t **result, int *context, void **handle);
sm_status_ret_code_t smc_icomm_respond_GetDeltaBegin(int fd, sm_value_t *result);
sm_status_ret_code_t smc_icomm_receive_GetDeltaBegin(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_request_GetDeltaProcess(int context, sm_value_t *params, sm_value_t **result);
sm_status_ret_code_t smc_icomm_respond_GetDeltaProcess(int fd, sm_value_t *result);
sm_status_ret_code_t smc_icomm_receive_GetDeltaProcess(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_GetDeltaRelease(int *context, void **handle);

sm_status_ret_code_t smc_icomm_request_PutDeltaBegin(uid_t uid, sm_value_t *params, sm_value_t **result, int *context, void **handle);
sm_status_ret_code_t smc_icomm_respond_PutDeltaBegin(int fd, sm_value_t *result);
sm_status_ret_code_t smc_icomm_receive_PutDeltaBegin(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_request_PutDeltaProcess(int context, sm_value_t *params, sm_value_t **result);
sm_status_ret_code_t smc_icomm_respond_PutDeltaProcess(int fd, sm_value_t *result);
sm_status_ret_code_t smc_icomm_receive_PutDeltaProcess(int fd, sm_value_t **params);
sm_status_ret_code_t smc_icomm_PutDeltaRelease(int *context, void **handle);

ICOMM_DECL(ListDirectory)
ICOMM_DECL(StatFile)
ICOMM_DECL(Delete)
ICOMM_DECL(ChMod)
ICOMM_DECL(MkDir)
#undef ICOMM_DECL

#endif /* __SMC_ICOMM_H */
