/* $Id: rsvn.h 2 2009-10-05 10:09:52Z mamonski $ */

/*
 * Copyright 2006-2009 QCG Project Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __SMC_RSVN_H
#define __SMC_RSVN_H

#include <stdbool.h>
#include <time.h>

#include <qcg-core/sm.h>
#include <qcg-core/odbc.h>
#include <qcg-core/list.h>

#include <stub/qcgcompStub.h>

typedef enum {
	SMC_RSVN_OK = SM_OK,
	SMC_RSVN_INTERNAL_FAULT,
	SMC_RSVN_DATABASE_FAULT,
	SMC_RSVN_NOT_FOUND,
	SMC_RSVN_INVALID_ARGUMENT
} smc_rsvn_ret_code_t;

struct smc_rsvn_s;

typedef const char *(*smc_rsvn_string_getter_function)(struct smc_rsvn_s *this);
typedef _ardl__ReservationDefinition *(*smc_rsvn_ardl_parsed_getter_function)(struct smc_rsvn_s *this);
typedef struct tm (*smc_rsvn_time_getter_function)(struct smc_rsvn_s *this);
typedef bool (*smc_rsvn_bool_getter_function)(struct smc_rsvn_s *this);

typedef void (*smc_rsvn_void_setter_function)(struct smc_rsvn_s *this);
typedef const char *(*smc_rsvn_string_setter_function)(struct smc_rsvn_s *this, const char *value);
typedef const char *(*smc_rsvn_ardl_parsed_setter_function)(struct smc_rsvn_s *this, _ardl__ReservationDefinition *value);
typedef struct tm (*smc_rsvn_time_setter_function)(struct smc_rsvn_s *this, struct tm value);

typedef sm_ret_code_t (*smc_rsvn_save_function)(struct smc_rsvn_s *this);

typedef struct smc_rsvn_s {
	smc_rsvn_string_getter_function get_id;
	smc_rsvn_string_getter_function get_drms_id;
	smc_rsvn_string_getter_function get_submitted_ardl;
	smc_rsvn_string_getter_function get_ardl;
	smc_rsvn_ardl_parsed_getter_function get_ardl_parsed;
	smc_rsvn_string_getter_function get_subject;
	smc_rsvn_string_getter_function get_mapped_user;
	smc_rsvn_time_getter_function get_submission_time;
	smc_rsvn_time_getter_function get_start_time;
	smc_rsvn_time_getter_function get_end_time;

	smc_rsvn_string_setter_function set_drms_id;
	smc_rsvn_string_setter_function set_submitted_ardl;
	smc_rsvn_string_setter_function set_ardl;
	smc_rsvn_ardl_parsed_setter_function set_ardl_parsed;
	smc_rsvn_string_setter_function set_subject;
	smc_rsvn_string_setter_function set_mapped_user;
	smc_rsvn_time_setter_function set_start_time;
	smc_rsvn_time_setter_function set_end_time;

	smc_rsvn_void_setter_function archive;
	smc_rsvn_void_setter_function terminate;
	smc_rsvn_bool_getter_function is_readonly;
	smc_rsvn_save_function save;
} *smc_rsvn_t;

sm_status_ret_code_t smc_rsvn_setup(sm_odbc_handle_t odbc_handle);
void smc_rsvn_cleanup(void);

smc_rsvn_t smc_rsvn_create(void);
void smc_rsvn_free(smc_rsvn_t *job);

sm_ret_code_t smc_rsvn_get(const char *id, smc_rsvn_t *result);
sm_ret_code_t smc_rsvn_get_by_drms_id(const char *drms_id, smc_rsvn_t *result);

sm_ret_code_t smc_rsvn_get_readonly(const char *id, smc_rsvn_t *result);
sm_ret_code_t smc_rsvn_get_by_drms_id_readonly(const char *drms_id, smc_rsvn_t *result);

sm_ret_code_t smc_rsvn_get_active_readonly(sm_list_t *result);

int smc_rsvn_count(void);

#endif /* __SMC_RSVN_H */
