/* $Id: xmlstructs_test.c 2 2009-10-05 10:09:52Z mamonski $ */

/*
 * Copyright 2006-2009 QCG Project Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <qcg-core/sm.h>
#include <qcg-core/logger.h>
#include <qcg-core/xmalloc.h>

#include <xmlstructs.h>

static const char *xml = "<jsdl:JobDefinition \
xmlns:jsdl=\"http://schemas.ggf.org/jsdl/2005/11/jsdl\" xmlns:jsdl-qcg-comp-ares=\"http://schemas.qoscosgrid.org/comp/2011/04/jsdl/ares\"><jsdl:JobDescription><jsdl:JobIdentification><jsdl:JobName>Pong_0</jsdl:JobName><jsdl:Description>QCG \
ProActive \
App</jsdl:Description></jsdl:JobIdentification><jsdl:Application><jsdl-hpcpa:HPCProfileApplication \
xmlns:jsdl-hpcpa=\"http://schemas.ggf.org/jsdl/2006/07/jsdl-hpcpa\"><jsdl-hpcpa:Executable>/opt/QCG/ProActive3.9/scripts/unix/qcg_pa_wrapper.sh</jsdl-hpcpa:Executable><jsdl-hpcpa:Argument>-Dproactive.useIPaddress=true</jsdl-hpcpa:Argument><jsdl-hpcpa:Argument>-Dqcg.proactive.qcgrmi.map=balboa.imb.uq.edu.au=balboa.imb.uq.edu.au:18001/192.168.42.61,192.168.42.62=www.colbud.hu:2234/qcg1.lri.fr,qcg2.lri.fr,qcg3.lri.fr,qcg4.lri.fr,qcg.lri.fr=qcg.lri.fr:4096/node2.qoscosgrid.man.poznan.pl=node2.qoscosgrid.man.poznan.pl:5096</jsdl-hpcpa:Argument><jsdl-hpcpa:Argument>-Dproactive.communication.protocol=rmiqcg</jsdl-hpcpa:Argument><jsdl-hpcpa:Argument>-cp</jsdl-hpcpa:Argument><jsdl-hpcpa:Argument>/home/usecase9/PingPong.jar</jsdl-hpcpa:Argument><jsdl-hpcpa:Argument>pl.psnc.qcg.MainApp</jsdl-hpcpa:Argument><jsdl-hpcpa:Argument>1</jsdl-hpcpa:Argument><jsdl-hpcpa:Input>pad</jsdl-hpcpa:Input><jsdl-hpcpa:Output>stdout_Pong_0</jsdl-hpcpa:Output><jsdl-hpcpa:Error>stderr_Pong_0</jsdl-hpcpa:Error><jsdl-hpcpa:Environment \
name=\"PROACTIVE_PROCESSES_ID\">Pong</jsdl-hpcpa:Environment><jsdl-hpcpa:Environment \
name=\"PROACTIVE_PNC_URL\">rmi://qcg.lri.fr:4098/PNC</jsdl-hpcpa:Environment><jsdl-hpcpa:Environment \
name=\"PROACTIVE_SESSION_ID\">58a72a21-5221-4609-9b46-4f689490fe8d_1219749354617</jsdl-hpcpa:Environment><jsdl-hpcpa:Environment \
name=\"PROACTIVE_MASTER_GROUP\">false</jsdl-hpcpa:Environment></jsdl-hpcpa:HPCProfileApplication></jsdl:Application> \
<jsdl:Resources><jsdl-qcg-comp-ares:Reservation><jsdl-qcg-comp-ares:ReservationIdentifier xmlns:wsa5=\"http://www.w3.org/2005/08/addressing\"><wsa5:Address>http://localhost:19000/</wsa5:Address><wsa5:ReferenceParameters><qcg-comp-ares:ReservationId xmlns:qcg-comp-ares=\"http://schemas.qoscosgrid.org/comp/2011/04/ares\">urn:uuid:9b7183a3-420e-45b7-863e-915dc380594d</qcg-comp-ares:ReservationId></wsa5:ReferenceParameters></jsdl-qcg-comp-ares:ReservationIdentifier></jsdl-qcg-comp-ares:Reservation></jsdl:Resources> \
<jsdl:DataStaging><jsdl:FileName>pad</jsdl:FileName><jsdl:CreationFlag>overwrite</jsdl:CreationFlag><jsdl:DeleteOnTermination>true</jsdl:DeleteOnTermination><jsdl:Source><jsdl:URI>cid:3c37b36e-ec33-4307-ad16-1b1dde1f8633</jsdl:URI></jsdl:Source></jsdl:DataStaging><jsdl:DataStaging><jsdl:FileName>stdout_Pong_0</jsdl:FileName><jsdl:CreationFlag>overwrite</jsdl:CreationFlag><jsdl:DeleteOnTermination>true</jsdl:DeleteOnTermination><jsdl:Target><jsdl:URI>stdout_Pong_0</jsdl:URI></jsdl:Target></jsdl:DataStaging><jsdl:DataStaging><jsdl:FileName>stderr_Pong_0</jsdl:FileName><jsdl:CreationFlag>overwrite</jsdl:CreationFlag><jsdl:DeleteOnTermination>true</jsdl:DeleteOnTermination><jsdl:Target><jsdl:URI>stderr_Pong_0</jsdl:URI></jsdl:Target></jsdl:DataStaging><jsdl:DataStaging><jsdl:FileName>PingPong.jar</jsdl:FileName><jsdl:CreationFlag>overwrite</jsdl:CreationFlag><jsdl:DeleteOnTermination>true</jsdl:DeleteOnTermination><jsdl:Source><jsdl:URI>cid:0a5bcb82-4ff8-4614-ab9e-49d528dbc29e</jsdl:URI></jsdl:Source></jsdl:DataStaging></jsdl:JobDescription></jsdl:JobDefinition>";

int
main()
{
	struct jsdl__JobDefinition_USCOREType jsdl;
	sm_alloc_tracked_t tracked;
	struct soap_dom_element *dom;

	assert(sm_library_setup() == SM_OK);
	assert(sm_log_init_str(NULL, "DEBUG") == SM_OK);

	tracked = sm_gsoap_buffer_get(xml, &jsdl, jsdl__JobDefinition_USCOREType, NULL, xmlstructs_namespaces);
	assert(tracked);

	assert(!strcmp("/opt/QCG/ProActive3.9/scripts/unix/qcg_pa_wrapper.sh", jsdl.jsdl__JobDescription->jsdl__Application->jsdl_hpcpa__HPCProfileApplication->jsdl_hpcpa__Executable->__item));

	assert(jsdl.jsdl__JobDescription->jsdl__Resources->jsdl_qcg_comp_ares__Reservation->ReservationIdentifier.ReferenceParameters->__size == 1);
	dom = jsdl.jsdl__JobDescription->jsdl__Resources->jsdl_qcg_comp_ares__Reservation->ReservationIdentifier.ReferenceParameters->__any;

	assert(dom->type == SOAP_TYPE__qcg_comp_ares__ReservationId);
	assert(!strcmp((const char *) dom->node, "urn:uuid:9b7183a3-420e-45b7-863e-915dc380594d"));

	sm_alloc_tracked_free(&tracked);

	sm_library_cleanup();

	return 0;
}
